@@ -54,7 +54,7 @@
 //#define DEBUG_MENU  1
 //#define DEBUG_CANDIDATES  1
 //#define DEBUG_SESSION  1
-//#define DEBUG_UIM  1
+//#define DEBUG_UIM  "3"
 //#define DEBUG_PRIME  1
 
 
@@ -124,7 +124,7 @@ NotificationCallback(CFNotificationCenterRef inCenter, void *inObserver,
                      CFDictionaryRef inUserInfo);
 
 static int
-ConvertKeyVal(int inKey);
+ConvertKeyVal(int inKey, int inMod);
 
 static int
 ConvertModifier(int inMod);
@@ -241,7 +241,7 @@ MUIMInitialize(ComponentInstance inComponentInstance,
     result = MUIMInitializeMessageReceiving();
   */
 
-#ifdef DEBUG_SESSION
+#if DEBUG_SESSION
   DEBUG_PRINT("MUIMInitialize()\n");
 #endif
 
@@ -258,12 +258,16 @@ MUIMInitialize(ComponentInstance inComponentInstance,
     }
   }
 
-#ifdef DEBUG_UIM
+#if DEBUG_UIM
   // Uim debug
-  setenv("LIBUIM_VERBOSE", "3", 1);
+  {
+    char num[16];
+    sprintf(num, "%d", DEBUG_UIM);
+    setenv("LIBUIM_VERBOSE", num, 1);
+  }
 #endif
 
-#ifdef DEBUG_PRIME
+#if DEBUG_PRIME
   // PRIME debug
   setenv("PRIME_DEBUG", "1", 1);
 #endif
@@ -297,11 +301,11 @@ MUIMInitialize(ComponentInstance inComponentInstance,
 void
 MUIMTerminate(ComponentInstance inComponentInstance)
 {
-#ifdef DEBUG_SESSION
+#if DEBUG_SESSION
     DEBUG_PRINT("MUIMTerminate()\n");
 #endif
 
-#ifdef ENABLE_HELPER
+#if ENABLE_HELPER
     UIMHelperClose();
 #endif
 
@@ -323,7 +327,7 @@ MUIMSessionOpen(ComponentInstance inComponentInstance,
   if (*outSessionHandle == nil) {
     *outSessionHandle =
       (MUIMSessionHandle) NewHandle(sizeof(MUIMSessionRecord));
-#ifdef DEBUG_SESSION
+#if DEBUG_SESSION
     //DEBUG_PRINT("MUIMSessionOpen() NewHandle=%p\n", *outSessionHandle);
 #endif
   }
@@ -378,7 +382,7 @@ MUIMSessionClose(MUIMSessionHandle inSessionHandle)
 {
   UInt32 i, j;
 
-#ifdef DEBUG_SESSION
+#if DEBUG_SESSION
   DEBUG_PRINT("MUIMSessionClose() inSessionHandle=%p\n",
               inSessionHandle);
 #endif
@@ -418,12 +422,12 @@ MUIMSessionActivate(MUIMSessionHandle inSessionHandle)
                                       smScriptKeys);
   SetScriptVariable (smJapanese, smScriptKeys, keyboardID);
 
-#ifdef DEBUG_SESSION
+#if DEBUG_SESSION
   DEBUG_PRINT("MUIMSessionActivate() gActiveSession=%p\n",
               gActiveSession);
 #endif
 
-#ifdef ENABLE_HELPER
+#if ENABLE_HELPER
   UIMCheckHelper();
   uim_prop_list_update(gUC);
   uim_prop_label_update(gUC);
@@ -436,12 +440,12 @@ MUIMSessionActivate(MUIMSessionHandle inSessionHandle)
 ComponentResult
 MUIMSessionDeactivate(MUIMSessionHandle inSessionHandle)
 {
-#ifdef DEBUG_SESSION
+#if DEBUG_SESSION
   DEBUG_PRINT("MUIMSessionDeactivate() gActiveSession=%p\n",
               gActiveSession);
 #endif
 
-#ifdef ENABLE_HELPER
+#if ENABLE_HELPER
   uim_helper_client_focus_out(gUC);
 #endif
 
@@ -499,7 +503,7 @@ MUIMSessionEvent(MUIMSessionHandle inSessionHandle, EventRef inEventRef)
     GetEventParameter(inEventRef, kEventParamKeyModifiers, typeUInt32, nil,
                       sizeof(modifiers), nil, &modifiers);
 
-#ifdef DEBUG_KEYEVENT
+#if DEBUG_KEYEVENT
     DEBUG_PRINT("MUIMSessionEvent() keycode=0x%lx, char=%c, charCode=0x%x, modifiers=0x%lx\n",
                 keyCode, charCode, charCode, modifiers);
 #endif
@@ -516,7 +520,7 @@ MUIMSessionEvent(MUIMSessionHandle inSessionHandle, EventRef inEventRef)
 MUIMSessionHandle
 MUIMGetActiveSession(void)
 {
-#ifdef DEBUG_SESSION
+#if DEBUG_SESSION
   DEBUG_PRINT("MUIMGetActiveSession()\n");
 #endif
 
@@ -536,7 +540,7 @@ MUIMHandleInput(MUIMSessionHandle inSessionHandle, UInt32 inKeycode,
   handled = FALSE;
   isScriptKey = FALSE;
 
-#ifdef DEBUG_KEYEVENT
+#if DEBUG_KEYEVENT
   DEBUG_PRINT("MUIMHandleInput() inCharCode=%02x inModifiers=%lx\n",
               inCharCode, inModifiers);
 #endif
@@ -553,11 +557,11 @@ MUIMHandleInput(MUIMSessionHandle inSessionHandle, UInt32 inKeycode,
   
   /* Then convert normal keys */
   if (key == 0)
-    key = ConvertKeyVal(inCharCode);
+    key = ConvertKeyVal(inCharCode, inModifiers);
 
   mod = ConvertModifier(inModifiers);
 
-#ifdef DEBUG_KEYEVENT
+#if DEBUG_KEYEVENT
   DEBUG_PRINT("MUIMHandleInput() key=0x%x mod=0x%x\n", key, mod);
 #endif
 
@@ -568,7 +572,7 @@ MUIMHandleInput(MUIMSessionHandle inSessionHandle, UInt32 inKeycode,
   if (!rv || isScriptKey)
     handled = TRUE;
 
-#ifdef DEBUG_KEYEVENT
+#if DEBUG_KEYEVENT
   DEBUG_PRINT("MUIMHandleInput() uim_press_key handled=%s\n",
               handled ? "true" : "false");
 #endif
@@ -579,7 +583,7 @@ MUIMHandleInput(MUIMSessionHandle inSessionHandle, UInt32 inKeycode,
 void
 MUIMUpdateShowHideKeyboardPaletteMenuItem(Boolean inIsHidden)
 {
-#ifdef DEBUG_CANDIDATES
+#if DEBUG_CANDIDATES
   DEBUG_PRINT("MUIMUpdateShowHideKeyboardPaletteMenuItem() inIsHidden=%d\n",
               inIsHidden);
 #endif
@@ -597,7 +601,7 @@ MUIMUpdateShowHideKeyboardPaletteMenuItem(Boolean inIsHidden)
 void
 MUIMUpdateShowHideSendEventPaletteMenuItem(Boolean inIsHidden)
 {
-#ifdef DEBUG_CANDIDATES
+#if DEBUG_CANDIDATES
   DEBUG_PRINT("MUIMUpdateShowHideSendEventPaletteMenuItem() inIsHidden=%d\n",
               inIsHidden);
 #endif
@@ -621,7 +625,7 @@ MUIMPencilMenuEventHandler(EventHandlerCallRef inEventHandlerCallRef,
   OSStatus result;
   HICommand command;
 
-#ifdef DEBUG_MENU
+#if DEBUG_MENU
   DEBUG_PRINT("MUIMPencilMenuEventHandler()\n");
 #endif
 
@@ -657,18 +661,17 @@ static void
 CreateUIMContext(CFStringRef inIMName)
 {
   const char *imName;
+  CFPropertyListRef imVal = NULL;
 
   if (inIMName) {
-    imName = CFStringGetCStringPtr(inIMName, kCFStringEncodingASCII);
+    imName = CFStringGetCStringPtr(inIMName, kCFStringEncodingMacRoman);
   }
   else {
-    CFPropertyListRef imVal;
-
     // Load the preferences
     imVal = CFPreferencesCopyAppValue(CFSTR("InputMethod"), CFSTR(kAppID));
     if (imVal && CFGetTypeID(imVal) == CFStringGetTypeID())
       imName = CFStringGetCStringPtr((CFStringRef) imVal,
-                                     kCFStringEncodingASCII);
+                                     kCFStringEncodingMacRoman);
     else
       imName = kDefaultIM;
   }
@@ -678,6 +681,9 @@ CreateUIMContext(CFStringRef inIMName)
   gUC = uim_create_context(gSessionList, "UTF-8", NULL, imName,
                            uim_iconv, UIMCommitString);
 
+  if (imVal)
+    CFRelease(imVal);
+
   if (!gUC)
     return;
 
@@ -686,7 +692,7 @@ CreateUIMContext(CFStringRef inIMName)
                      UIMPreeditPushback,
                      UIMPreeditUpdate);
 
-#ifdef ENABLE_HELPER
+#if ENABLE_HELPER
   UIMCheckHelper();
   uim_set_prop_list_update_cb(gUC, UIMUpdatePropList);
   uim_set_prop_label_update_cb(gUC, UIMUpdatePropLabel);
@@ -739,16 +745,28 @@ NotificationCallback(CFNotificationCenterRef inCenter, void *inObserver,
   }
 }
 
+/*
+ * key codes are defined in
+ * /System/Library/Frameworks/Carbon.framework/Frameworks/HIToolbox.framework/Headers/Events.h
+ */
+
 static int
-ConvertKeyVal(int inKey)
+ConvertKeyVal(int inKey, int inMod)
 {
-#ifdef DEBUG_KEYEVENT
-  DEBUG_PRINT("ConvertKeyVal() inKey=%02x\n", inKey);
+  int key = inKey;
+
+#if DEBUG_KEYEVENT
+  DEBUG_PRINT("ConvertKeyVal() inKey=0x%02x inMod=0x%02x\n", inKey, inMod);
 #endif
 
-  if (inKey >= 0x01 && inKey <= 0x1a)
-    return inKey + 0x60;
-  return inKey;
+  if (inKey >= 0x01 && inKey <= 0x1a) {
+    if (inMod & shiftKey)
+      key += 0x40;
+    else
+      key += 0x60;
+  }
+
+  return key;
 }
 
 static int
@@ -756,14 +774,16 @@ ConvertModifier(int inMod)
 {
   int modifier = 0;
 
-#ifdef DEBUG_KEYEVENT
-  DEBUG_PRINT("ConvertModifier() inMod=%lx\n", inMod);
+#if DEBUG_KEYEVENT
+  DEBUG_PRINT("ConvertModifier() inMod=0x%02x\n", inMod);
 #endif
 
   if (inMod & shiftKey)
     modifier += UMod_Shift;
   if (inMod & controlKey)
     modifier += UMod_Control;
+  if (inMod & optionKey)
+    modifier += UMod_Alt;
 
   return modifier;
 }
